from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django import forms

from evotecore import widgets

from .models import Election


class ElectoralCalendarForm(forms.ModelForm):
    """ """

    error_messages = {
        'proposition_end_date_invalid':
            _("Date must be after the begin of the period for the submission"
              " of applications."),
        'roll_end_date_invalid':
            _("Date must be after the publication of the provisional "
              'electoral roll.'),
        'voting_begin_date_invalid':
            _("Date must be after the publication of the definitive "
              "electoral roll."),
        'voting_end_date_invalid':
            _("Date must be after the begin of the voting time."),
        'minute_publication_date_invalid':
            _("Date must be after the end of the voting time."),
        'minute_begin_date_invalid':
            _("Date must be after the end of the voting time."),
        'minute_end_date_invalid':
            _("Date must be after the start of the approval period.")
    }

    class Meta:
        model = Election
        fields = [
            'proposition_begin_date', 'proposition_end_date',
            'roll_begin_date', 'roll_end_date', 'voting_begin_date',
            'voting_end_date', 'minute_publication_date',
            'minute_begin_date', 'minute_end_date'
        ]
        widgets = {
            'proposition_begin_date': widgets.DateTimeInput(),
            'proposition_end_date': widgets.DateTimeInput(),
            'roll_begin_date': widgets.DateTimeInput(),
            'roll_end_date': widgets.DateTimeInput(),
            'voting_begin_date': widgets.DateTimeInput(),
            'voting_end_date': widgets.DateTimeInput(),
            'minute_publication_date': widgets.DateTimeInput(),
            'minute_begin_date': widgets.DateTimeInput(),
            'minute_end_date': widgets.DateTimeInput()
        }

    def __init__(self, *args, **kwargs):
        super(ElectoralCalendarForm, self).__init__(*args, **kwargs)
        # disable date field if date already surpassed
        fields_date_check = ['proposition_begin_date', 'proposition_end_date',
                             'roll_begin_date', 'roll_end_date']
        if self.instance and self.instance.state > self.instance.State.OPEN:
            now = timezone.now()
            for field_name in fields_date_check:
                value = getattr(self.instance, field_name)
                self.fields[field_name].disabled = bool(value < now)

    def clean_proposition_end_date(self):
        proposition_end_date = self.cleaned_data.get('proposition_end_date')
        proposition_begin_date = self.cleaned_data.get(
            'proposition_begin_date', None)
        if proposition_end_date and proposition_begin_date:
            if proposition_end_date <= proposition_begin_date:
                raise forms.ValidationError(
                    self.error_messages['proposition_end_date_invalid'])
        return proposition_end_date

    def clean_roll_end_date(self):
        roll_end_date = self.cleaned_data.get('roll_end_date')
        roll_begin_date = self.cleaned_data.get('roll_begin_date', None)
        if roll_end_date and roll_begin_date:
            if roll_end_date <= roll_begin_date:
                raise forms.ValidationError(
                    self.error_messages['roll_end_date_invalid'])
        return roll_end_date

    def clean_voting_begin_date(self):
        voting_begin_date = self.cleaned_data.get('voting_begin_date')
        proposition_end_date = self.cleaned_data.get(
            'proposition_end_date', None)
        roll_end_date = self.cleaned_data.get('roll_end_date', None)
        if voting_begin_date and roll_end_date and proposition_end_date:
            if voting_begin_date < roll_end_date or \
                    voting_begin_date < proposition_end_date:
                raise forms.ValidationError(
                    self.error_messages['voting_begin_date_invalid'])
        return voting_begin_date

    def clean_voting_end_date(self):
        voting_end_date = self.cleaned_data.get('voting_end_date', None)
        voting_begin_date = self.cleaned_data.get('voting_begin_date')
        if voting_end_date and voting_begin_date:
            if voting_end_date <= voting_begin_date:
                raise forms.ValidationError(
                    self.error_messages['voting_end_date_invalid'])
        return voting_end_date

    def clean_minute_publication_date(self):
        minute_publication_date = self.cleaned_data.get(
            'minute_publication_date')
        voting_end_date = self.cleaned_data.get('voting_end_date', None)
        if minute_publication_date and voting_end_date:
            if minute_publication_date < voting_end_date:
                raise forms.ValidationError(
                    self.error_messages['minute_publication_date_invalid'])
        return minute_publication_date

    def clean_minute_begin_date(self):
        minute_begin_date = self.cleaned_data.get('minute_begin_date')
        voting_end_date = self.cleaned_data.get('voting_end_date', None)
        if minute_begin_date and voting_end_date:
            if minute_begin_date < voting_end_date:
                raise forms.ValidationError(
                    self.error_messages['minute_begin_date_invalid'])
        return minute_begin_date

    def clean_minute_end_date(self):
        minute_end_date = self.cleaned_data.get('minute_end_date')
        minute_begin_date = self.cleaned_data.get('minute_begin_date', None)
        if minute_end_date and minute_begin_date:
            if minute_end_date <= minute_begin_date:
                raise forms.ValidationError(
                    self.error_messages['minute_end_date_invalid'])
        return minute_end_date
