from django.utils import timezone
from django.test import TestCase
from django.apps import apps

from eVotUM.Cripto import eccblind

from evotevotings.elections.utils import get_extra_election_data

from evotevotings.admin.models import SecretEntry
from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.admin.utils import (get_certificates,
                                      not_expired_certificate,
                                      validate_certificate,
                                      get_secret_entry)


# Helper Class
class ServiceTestUtils(ElectionsTestUtils):
    """ Helper functions class """

    def get_election_model(self):
        return apps.get_model("evoteelections.Election")  # noqa

    def generate_fake_election(self):
        now = timezone.now()
        vbegin = now - timezone.timedelta(days=1)
        vend = now + timezone.timedelta(days=1)
        status = self.get_election_model().State.IN_TO_VOTE
        # Generate fixtures
        votesys_cert_id = SecretEntry.Identifier.VOTING_CERT
        self.election = self.create_election(
            voting_begin_date=vbegin, voting_end_date=vend,
            state=status,
            certificate=get_secret_entry(votesys_cert_id))
        self.create_candidates(4,
                               election=self.election)
        self.voter = self.create_voter(
            election=self.election
        )
        self.election_data = get_extra_election_data(
            self.election.pk,
            self.voter.user_id
        )
        self.certificates = get_certificates()
        # Initialize the blind signature
        (init_component, pr_component) = eccblind.initSigner()
        self.init_component = init_component
        self.pr_component = pr_component
        self.vote_bulletin = "DUMMY VOTE"


class ValidCertificateTestCase(TestCase):
    fixtures = ['users.json', 'certs.json']


class ExpiredRootCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_root.json']
    # Setup the Test Case

    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        root_cert = self.certificates['root']
        expired = not_expired_certificate(root_cert)
        self.assertFalse(expired)


class ExpiredVotingCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_voting.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['votesys']
        expired = not_expired_certificate(cert)
        self.assertFalse(expired)


class UnchainedVotingCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""
    fixtures = ['users.json', 'certs_unchained_voting.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['votesys']
        root_cert = self.certificates['root']
        chain = validate_certificate(cert, root_cert)
        self.assertEqual(chain, -1)


class ExpiredValidatorCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_exp_vs.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['vs']
        expired = not_expired_certificate(cert)
        self.assertFalse(expired)


class UnchainedValidorCertificateTestCase(TestCase, ServiceTestUtils):
    """ Class to test Expired Root certificate"""

    fixtures = ['users.json', 'certs_unchained_vs.json']

    # Setup the Test Case
    def setUp(self):
        # Create Election
        # Create ElectionVoter
        # Generate Candidates
        # Generate Signature
        self.generate_fake_election()

    def test_invalid_certificate(self):
        cert = self.certificates['vs']
        root_cert = self.certificates['root']
        chain = validate_certificate(cert, root_cert)
        self.assertEqual(chain, -1)
